#include <math.h>
#include <rgb.hpp>

// CTOR
Color::RGB::RGB() : r(0), g(0), b(0) {}
Color::RGB::RGB(const uint8_t& iR, 
		const uint8_t& iG,
		const uint8_t& iB) :
  r(iR), g(iG), b(iB)
{}
Color::RGB::RGB(const Color::RGB& iC) :
  r(iC.r), g(iC.g), b(iC.b)
{}
// DTOR
Color::RGB::~RGB() {}

// operators
Color::RGB& Color::RGB::operator= (const Color::RGB& iC) {
  r = iC.r;
  g = iC.g;
  b = iC.b;
  return *this;
}
Color::RGB& Color::RGB::operator+=(const Color::RGB& iC) {
  r += iC.r;
  g += iC.g;
  b += iC.b;
  return *this;
}
Color::RGB& Color::RGB::operator-=(const Color::RGB& iC) {
  r -= iC.r;
  g -= iC.g;
  b -= iC.b;
  return *this;
}
Color::RGB& Color::RGB::operator*=(const Color::RGB& iC) {
  r *= iC.r;
  g *= iC.g;
  b *= iC.b;
  return *this;
}
Color::RGB& Color::RGB::operator*=(const uint8_t& iS) {
  r *= iS;
  g *= iS;
  b *= iS;
  return *this;
}
Color::RGB& Color::RGB::operator/=(const Color::RGB& iC) {
  r /= iC.r;
  g /= iC.g;
  b /= iC.b;
  return *this;
}
Color::RGB& Color::RGB::operator/=(const uint8_t& iS) {
  r /= iS;
  g /= iS;
  b /= iS;
  return *this;
}

Color::RGB Color::operator+ (const Color::RGB& iC0, const Color::RGB& iC1) {
  return Color::RGB(iC0.r + iC1.r,
		    iC0.g + iC1.g,
		    iC0.b + iC1.b);
}

Color::RGB Color::operator- (const Color::RGB& iC0, const Color::RGB& iC1) {
  return Color::RGB(iC0.r - iC1.r,
		    iC0.g - iC1.g,
		    iC0.b - iC1.b);
}

Color::RGB Color::operator* (const Color::RGB& iC0, const Color::RGB& iC1) {
  return Color::RGB(iC0.r * iC1.r,
		    iC0.g * iC1.g,
		    iC0.b * iC1.b);
}

Color::RGB Color::operator* (const Color::RGB& iC, const uint8_t& iS) {
  return Color::RGB(iC.r * iS,
		    iC.g * iS,
		    iC.b * iS);
}

Color::RGB Color::operator* (const uint8_t& iS, const Color::RGB& iC) {
  return Color::RGB(iS * iC.r,
		    iS * iC.g,
		    iS * iC.b);
}

Color::RGB Color::operator/ (const Color::RGB& iC0, const Color::RGB& iC1) {
  return Color::RGB(iC0.r / iC1.r,
		    iC0.g / iC1.g,
		    iC0.b / iC1.b);
}

Color::RGB Color::operator/ (const Color::RGB& iC, const uint8_t& iS) {
  return Color::RGB(iC.r / iS,
		    iC.g / iS,
		    iC.b / iS);
}


std::ostream& Color::operator<< (std::ostream& iOut, const Color::RGB& iC) {
  iOut << static_cast<int>(iC.r) << ' ' << static_cast<int>(iC.g) << ' ' << static_cast<int>(iC.b);
  return iOut;
}

// Euclidian distance between 2 colors
uint32_t Color::euclidian(const Color::RGB& iC0, const Color::RGB& iC1) {
  return static_cast<uint32_t>(sqrt(((iC0.r - iC1.r) * (iC0.r - iC1.r)) + 
				   ((iC0.g - iC1.g) * (iC0.g - iC1.g)) +
				   ((iC0.b - iC1.b) * (iC0.b - iC1.b))));
}
// Manhattan distance between 2 colors
uint32_t Color::manhattan(const Color::RGB& iC0, const Color::RGB& iC1) {
  return static_cast<uint32_t>(abs(iC0.r - iC1.r) + fabs(iC0.g - iC1.g) + fabs(iC0.b - iC1.b));
}
